package com.sybase.sup.samples.objectapi;
/*
 * Created on Jul 20, 2010
 *
 * Copyright (c) Sybase, Inc. 2010   
 * All rights reserved.                                    
 */

import java.util.Vector;
import net.rim.device.api.system.Characters;
import net.rim.device.api.ui.DrawStyle;
import net.rim.device.api.ui.Graphics;
import net.rim.device.api.ui.UiApplication;
import net.rim.device.api.ui.XYRect;
import net.rim.device.api.ui.component.BasicEditField;
import net.rim.device.api.ui.component.KeywordFilterField;
import net.rim.device.api.ui.component.ListField;
import net.rim.device.api.ui.component.ListFieldCallback;
import SUP101.Customer;
import SUP101.SUP101DB;
import com.sybase.collections.ObjectList;

import com.sybase.mobile.*;
import com.sybase.persistence.SynchronizationGroup;

/**
 * @author bdeng
 * 
 * Customer sample main application.
 * 
 * Note: For the sake of simplicity, this sample application may not leverage
 * resource bundles and resource strings.  However, it is STRONGLY recommended
 * that application developers make use of the localization features available
 * within the BlackBerry development platform to ensure a seamless application
 * experience across a variety of languages and geographies.  For more information
 * on localizing your application, please refer to the BlackBerry Java Development
 * Environment Development Guide.
 */

public final class CustomerSample extends UiApplication
{
    private KeywordFilterField _keywordFilterField;
    private CustomerList       _customerList;
    private Vector             _customers;

    /**
     * Entry point for application.
     * 
     * @param args Command line arguments (not used).
     */
    public static void main(String[] args)
    {
        // Create a new instance of the application
        CustomerSample app = new CustomerSample();

        // Make the currently running thread the application's event
        // dispatch thread and begin processing events.
        app.enterEventDispatcher();
    }

    /**
     * Constructor
     */
    public CustomerSample()
    {
        // Create an instance of KeywordFilterField class.
        _keywordFilterField = new KeywordFilterField();
        _keywordFilterField.setCallback(new MyListFieldCallback());
        
        // Add our list to a KeywordFilterField object.
        _customerList = new CustomerList();
        _keywordFilterField.setSourceList(_customerList, _customerList);

        // We're providing a customized edit field for
        // the KeywordFilterField.
        CustomKeywordField customSearchField = new CustomKeywordField();
        _keywordFilterField.setKeywordField(customSearchField);

        // Create main screen.
        CustomerSampleScreen screen = new CustomerSampleScreen(this);

        // We need to explicitly add the search/title field via
        // MainScreen.setTitle().
        screen.setTitle(_keywordFilterField.getKeywordField());

        // Add our KeywordFilterField to the screen and push the screen
        // onto the stack.
        screen.add(_keywordFilterField);
        pushScreen(screen);
    }

    /**
     * Refreshes the data.
     */
    public void refreshData()
    {
        _customers = loadData();
        _customerList.loadFrom(_customers.elements());
    }

    /**
     * Method to access the KeywordFilterField object.
     * 
     * @return This applications KeywordFilterField.
     */
    KeywordFilterField getKeywordFilterField()
    {
        return _keywordFilterField;
    }

    /**
     * Method populates and returns a vector of Customer objects containing data
     * read from data source.
     * 
     * @return A vector containing Customer objects.
     */
    private Vector loadData()
    {
        ObjectList customers = Customer.findAll();
        Vector customerVec = new Vector(customers.size());

        int size = customers.count();
        for (int i = 0; i < size; i++)
        {
            customerVec.addElement(customers.elementAt(i));
        }

        return customerVec;
    }

    /**
     * Inner Class: A custom keyword input field for the KeywordFilterField. We
     * want to prevent a save dialog from being presented to the user when
     * exiting the application as the ability to persist data is not relevent to
     * this application. We are also using the paint() method to customize the
     * appearance of the cursor in the input field.
     */
    final static class CustomKeywordField extends BasicEditField
    {
        // Contructor
        CustomKeywordField()
        {
            // Custom style.
            super(USE_ALL_WIDTH | NON_FOCUSABLE | NO_LEARNING | NO_NEWLINE);

            setLabel("Search: ");
        }

        /**
         * Intercepts ESCAPE key.
         * 
         * @see net.rim.device.api.ui.component.TextField#keyChar(char,int,int)
         */
        protected boolean keyChar(char ch, int status, int time)
        {
            switch (ch)
            {
                case Characters.ESCAPE:
                    // Clear keyword.
                    if ( super.getTextLength() > 0 )
                    {
                        setText("");
                        return true;
                    }
            }
            return super.keyChar(ch, status, time);
        }

        /**
         * Overriding super to add custom painting to our class.
         * 
         * @see net.rim.device.api.ui.Field#paint(Graphics)
         */
        protected void paint(Graphics graphics)
        {
            super.paint(graphics);

            // Draw caret.
            getFocusRect(new XYRect());
            drawFocus(graphics, true);
        }
    }
    
    private class MyListFieldCallback implements ListFieldCallback {

        public void drawListRow(ListField listField, Graphics graphics, int index, int y, int width) {
            Customer customer = (Customer)((KeywordFilterField)listField).getElementAt(index);
            
            customer = CustomerDBCallback.getInstace().reLoadIfNeeded(customer);
            // 4 pixel padding from left and right
            graphics.drawText(customer.getFname() + " " + customer.getLname(), 0, y, DrawStyle.HDEFAULT, width);
        }

        public Object get(ListField listField, int index) { 
            return null; 
        }

        public int getPreferredWidth(ListField listField) { 
            return 0; 
        }

        public int indexOfList(ListField listField, String prefix, int start) { 
            return 0; 
        }
    }
}
