package com.sybase.sup.samples.objectapi;

import SUP101.SUP101DB;

import com.sybase.collections.ObjectList;
import com.sybase.mobile.*;
import com.sybase.persistence.LoginCredentials;
import com.sybase.persistence.SynchronizationGroup;

/**
 * A wrapper class for object API
 */
public class ObjectAPIUtil
{
    private static String USERNAME = "supAdmin";
    private static String PASSWORD = "s3pAdmin";
    private static String HOST = "example-host";
    private static int PORT = 5001;
    private static int TIMEOUT = 600;
    
    /**
     * Initialize the application. Including
     * 1. set application id
     * 2. register the package callback handler
     * 3. set up connection settings
     * 4. register application (application starts first time) or start connection
     * 5. enable target change notification
     */
    public static void initialize()
    {
        Application app = Application.getInstance();
        app.setApplicationIdentifier("SUP101"); 
        
        SUP101DB.registerCallbackHandler(CustomerDBCallback.getInstace()); //Register callback. Must be done
        SUP101DB.setApplication(app);
        ConnectionProperties  connProps = app.getConnectionProperties();
        LoginCredentials loginCredentials = new LoginCredentials(USERNAME,PASSWORD);
        
        connProps.setLoginCredentials(loginCredentials);
        connProps.setServerName(HOST);
        connProps.setPortNumber(PORT);

        if( app.getRegistrationStatus() != RegistrationStatus.REGISTERED )
        {            
            app.registerApplication(TIMEOUT);
        }
        else
        {
            app.startConnection(TIMEOUT);
        }
        
        if ( !SUP101DB.isSynchronized("default") )
        {
            SUP101DB.disableChangeLog();
            SUP101DB.synchronize();
            SynchronizationGroup sg = SUP101DB.getSynchronizationGroup("default");
            sg.setEnableSIS(true);
            sg.save();
            SUP101DB.synchronize();
        }
        
        SUP101DB.enableChangeLog();
    }
    
    /**
     * Synchronize the package in background.
     * It is an asynchronous call
     */
    public static void beginSynchronize()
    {
        ObjectList syncGroups = new ObjectList();
        syncGroups.add(SUP101DB.getSynchronizationGroup("default"));
        SUP101DB.beginSynchronize(syncGroups, "");
    }
}
